/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    TaskSpaceActiveImpedanceControl::ArmarXObjects::NJointTaskSpaceImpedanceController
 * @author     zhou ( you dot zhou at kit dot edu )
 * @date       2018
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#ifndef _ARMARX_LIB_TaskSpaceActiveImpedanceControl_NJointTaskSpaceImpedanceController_H
#define _ARMARX_LIB_TaskSpaceActiveImpedanceControl_NJointTaskSpaceImpedanceController_H

#include <RobotAPI/components/units/RobotUnit/NJointControllers/NJointController.h>
#include <VirtualRobot/Robot.h>
#include <RobotAPI/components/units/RobotUnit/RobotUnit.h>
#include <RobotAPI/components/units/RobotUnit/ControlTargets/ControlTarget1DoFActuator.h>
#include <RobotAPI/components/units/RobotUnit/SensorValues/SensorValue1DoFActuator.h>
#include <VirtualRobot/IK/DifferentialIK.h>

#include <RobotAPI/interface/units/RobotUnit/TaskSpaceActiveImpedanceControl.h>

namespace armarx
{
    class NJointTaskSpaceImpedanceControllerControlData
    {
    public:
        Eigen::VectorXf desiredPose;
        Eigen::VectorXf desiredTwist;
    };
    /**
    * @defgroup Library-NJointTaskSpaceImpedanceController NJointTaskSpaceImpedanceController
    * @ingroup Library-RobotUnit-NJointControllers
    * A description of the library NJointTaskSpaceImpedanceController.
    *
    * @class NJointTaskSpaceImpedanceController
    * @ingroup Library-NJointTaskSpaceImpedanceController
    * @brief Brief description of class NJointTaskSpaceImpedanceController.
    *
    * Detailed description of class NJointTaskSpaceImpedanceController.
    */
    class NJointTaskSpaceImpedanceController : public NJointControllerWithTripleBuffer<NJointTaskSpaceImpedanceControllerControlData>, public NJointTaskSpaceImpedanceControlInterface
    {
    public:
        using ConfigPtrT = NJointTaskSpaceImpedanceControlConfigPtr;

        NJointTaskSpaceImpedanceController(const RobotUnitPtr& robotUnit, const NJointControllerConfigPtr& config, const VirtualRobot::RobotPtr&);


        std::string getClassName(const Ice::Current&) const override
        {
            return "TaskSpaceImpedanceController";
        }

        // NJointController interface
        void rtRun(const IceUtil::Time& sensorValuesTimestamp, const IceUtil::Time& timeSinceLastIteration) override;

    protected:
        void onPublish(const SensorAndControl&, const DebugDrawerInterfacePrx&, const DebugObserverInterfacePrx&) override;

        void setTarget(const Ice::FloatSeq&, const Ice::Current&) override;
    private:
        Eigen::Vector3f desiredPosition;
        Eigen::Quaternionf desiredQuaternion;
        Eigen::VectorXf desiredTwist;
        VirtualRobot::DifferentialIKPtr ik;
        Eigen::Vector3f kpos;
        Eigen::Vector3f kori;
        Eigen::Vector3f dpos;
        Eigen::Vector3f dori;

        Eigen::VectorXf desiredJointPosition;

        float knull, dnull;

        VirtualRobot::RobotNodePtr tcp;

        std::vector<const SensorValue1DoFActuatorTorque*> torqueSensors;
        std::vector<const SensorValue1DoFActuatorVelocity*> velocitySensors;
        std::vector<const SensorValue1DoFActuatorPosition*> positionSensors;

        std::vector<ControlTarget1DoFActuatorTorque*> targets;

        struct NJointTaskSpaceImpedanceControllerDebugInfo
        {
            StringFloatDictionary desired_torques;
            float desiredForce_x;
            float desiredForce_y;
            float desiredForce_z;
            float tcpDesiredTorque_x;
            float tcpDesiredTorque_y;
            float tcpDesiredTorque_z;

            float tcpDesiredAngularError_x;
            float tcpDesiredAngularError_y;
            float tcpDesiredAngularError_z;

            float currentTCPAngularVelocity_x;
            float currentTCPAngularVelocity_y;
            float currentTCPAngularVelocity_z;

            float currentTCPLinearVelocity_x;
            float currentTCPLinearVelocity_y;
            float currentTCPLinearVelocity_z;

            float quatError;
            float posiError;
        };
        TripleBuffer<NJointTaskSpaceImpedanceControllerDebugInfo> debugDataInfo;

        float torqueLimit;
        std::vector<std::string> jointNames;


        // NJointController interface
    protected:
        void rtPreActivateController() override;
    };

}

#endif
